#!/bin/bash

# CC BY-NC-SA 4.0 Felix Baumann
# https://creativecommons.org/licenses/by-nc-sa/4.0
# 2017
# Script is provided as supplemental material
# for the article F. W. Baumann and D. Roller "Overview of German Additive Manufacturing Companies"

# This script scrapes the website of the central registry court information system
# for a specific company name and parses the official information.
# Furthermore, all available documents on the company are downloaded
# and stored locally. Only documents that do not incur costs
# are considered.

LANG=de_DE.UTF-8

function parse_ut()
{
	CROP=$( grep -A 70 "Unternehmensträgerdaten" ${1} | sed -e "s/[\t ]\{2,\}/ /g" | tr -d "\r" )
	RECHTSFORM=$( echo "${CROP}" | grep -A 1 "Rechtsform:" | tail -1l | sed -e "s/<[^>]*>//g;s/&nbsp;//g" | tr -d "\n" ) # | perl -MHTML::Entities -pe 'decode_entities($_);' |
	KAPITAL=$( echo "${CROP}" | grep -A 5 "Kapital:" | grep -v "Kapital:" | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr -d "\n" | sed -e "s/[\t ]*$//;s/,00 / /;s/\.//g" )
	ENTRY_DATE=$( echo "${CROP}" | grep -A 3 "Eintragsdatum:" | tail -1l | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr -d "\n " | awk -F"." '{ print $3"-"$2"-"$1 }' )
	DEL_DATE=$( echo "${CROP}" | grep -A 3 "Löschdatum:" | tail -1l | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr -d "\n " | awk -F"." '{ print $3"-"$2"-"$1 }' | sed -e "s/--/N\/A/" )
	#BILANZ=$( echo "${CROP}" | grep -A 3 "Bilanz vorhanden:" | tail -1l | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr -d "\n" )

	ADDR=$( echo "${CROP}" | grep -A 5 "Anschrift (ohne Gewähr):" | tail -4l | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr "\n" "," | sed -e "s/,,/,/g" | perl -MHTML::Entities -pe 'decode_entities($_);' )

	echo "\"${RECHTSFORM:-N/A}\";${KAPITAL:-N/A};${ENTRY_DATE:-N/A};${DEL_DATE:-N/A};${BILANZ:-N/A};\"${ADDR:-N/A}\"" | sed -e "s/[\t ]\{2,\}/ /g;s/^ //;s/,$//;s/; /;/g;s/, , /, /g"  # ;s/;;/;N\/A;/g"
}


TDIR=$( mktemp -d )

BASE_URL="https://www.handelsregister.de"
URL="${BASE_URL}/rp_web/search.do"
REF="https://www.handelsregister.de/rp_web/mask.do?Typ=n"
UA="Mozilla/5.0 (X11; Linux x86_64; rv:52.0)"

#HASH=$( echo "${1}" | md5sum )

SEARCH=$( echo "${1}" | sed -e "s/ /+/g" )


RESP=$( wget -O - -q --post-data="suchTyp=n&schlagwoerter=${SEARCH}&schlagwortOptionen=3&ergebnisseProSeite=10&btnSuche=Find" --keep-session-cookies --save-cookies ~/.wget.cookies --user-agent="${UA}" "${URL}" | perl -C -MHTML::Entities -pe 'decode_entities($_);' 2>/dev/null )
INFO=$( echo "${RESP}" | tr "\n" " " | sed -e "s/\r//g;s/Datenzeile/\n<Datenzeile/g;s/[\t ]\{2,\}/ /g" | grep -m 2 "Datenzeile"| sed -e "s/<[^>]*//g" )

COURT=$( echo "${INFO}" | head -1l | awk -F">" '{ print $5 }' | sed -e "s/^[\t ]*//g;s/[\t ]*$//g" )
NAME=$(  echo "${INFO}" | tail -1l | awk -F">" '{ print $6 }' | sed -e "s/^[\t ]*//g;s/[\t ]*$//g" )

echo -n "${NAME:-N/A};${COURT:-N/A};" | tr -d "\n"


UT=$( echo "${RESP}" | egrep -m 1 "doctyp=UT" | awk -F"\"" '{ print $2 }' | sed -e "s/amp;//g" )
VO=$( echo "${RESP}" | egrep -m 1 "doctyp=HB" | awk -F"\"" '{ print $2 }' | sed -e "s/amp;//g" )

UT_DATA=$( wget -O - -q --user-agent="${UA}" --referer="${URL}" --keep-session-cookies --load-cookies ~/.wget.cookies "${BASE_URL}${UT}" )

SEARCH=$( echo "${SEARCH}" | sed -e "s/\//+/g" )
echo "${UT_DATA}" > ${TDIR}/${SEARCH}_ut.htm
parse_ut "${TDIR}/${SEARCH}_ut.htm"

VO_DATA=$( wget -O - -q --user-agent="${UA}" --referer="${URL}" --keep-session-cookies --load-cookies ~/.wget.cookies "${BASE_URL}${VO}" | grep "direct-download.do" )
VO_DATA_LINKS=$( echo "${VO_DATA}" | grep "direct-download" | awk -F"\"" '{ print $6 }' | grep -v "^$" | uniq )

c=0
for l in ${VO_DATA_LINKS}; do
	VO_PUB=$( wget -O - -q --user-agent="${UA}" --referer="${BASE_URL}${VO}" --keep-session-cookies --load-cookies ~/.wget.cookies "${BASE_URL}/rp_web/${l}" )
	echo "${VO_PUB}" > ${TDIR}/${SEARCH}_vo_pub_${c}.htm
	echo "${VO_PUB}" | sed -e "s/<[^>]*>//g" | tr -d "\r" | sed -e "s/[\t ]\{2,\}/ /g" | grep -v "^[\t ]*$" > ${TDIR}/${SEARCH}_vo_pub_${c}.txt
	let c+=1
done
echo -e "TDIR\t${TDIR}"


