#!/bin/bash

# CC BY-NC-SA 4.0 Felix Baumann
# https://creativecommons.org/licenses/by-nc-sa/4.0
# 2017
# Script is provided as supplemental material
# for the article F. W. Baumann and D. Roller "Overview of German Additive Manufacturing Companies"

# This script scrapes the website of "wer liefert was?" by
# searching for a specific company name and the crawling through the list,
# formatting the output to CSV

LANG=de_DE.UTF-8

COMP=${1}

LOG=[ SPECIFY BASE DIRECTORY HERE]/wlw-search.txt

if [ ! -f ${LOG} ]; then
	touch ${LOG}
fi

echo "[$( date +"%Y-%m-%d %H:%M:%S" )] Found $( wc -l ${LOG} | awk '{ print $1 }' ) companies in ${LOG}" > /dev/stderr


function html_page_title()
{
	H_TITLE=$( wget -qO- "${1}" | hxselect -s '\n' -c 'title' 2>/dev/null | recode html...utf8 )
	echo "\"${H_TITLE:-N/A}\""
}

function detail_view()
{
	C=$( wget -O - -q --user-agent="Research Bot v1.1" "${1}" | perl -C -MHTML::Entities -pe 'decode_entities($_);' 2>/dev/null )
	
	MA=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -A 2 "Mitarbeiteranzahl:" | tail -1l | sed -e "s/<[^>]*>//;s/ - /--/" )
	EMAIL=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -m 1 "mailto" | awk -F"\"" '{ print $30 }' | awk -F"?" '{ print $1 }' | sed -e "s/mailto://g"| rev )
	GL=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -A 10 "Leitende" | tail -1l | sed -e "s/<[^>]*>//;s/Herr /(m)/g;s/Frau /(f)/g" | sed -e "s/\(([mf])\)\([^,;]*\)/\2\1/g" )
	#DESC=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -A 10 "Unternehmen" | grep -A 1 "panel-body" | tail -1l )
	DESC=$( echo "${C}" | tr "\r\n" " " | sed -e "s/<\/article>/\n/g;s/panel-title/\n</g" | grep  "Unternehmen" | head -1l | sed -e "s/<[^>]*>/ /g;s/ Unternehmen  //g;s/ mehr anzeigen //g;s/ weniger anzeigen //g" | sed -e "s/^[\t ]*//g;s/[\t ]*$//g" | egrep -v "(Partner der B2B-Plattform|transactionName)" ) #| perl -MHTML::Entities -pe 'decode_entities($_); )
	CAT=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -A 17 "panel-listing" | sed -e "s/<[^>]*>//g" | grep -v "^$" | tr "\n" "," | sed -e "s/,--//g;s/ ,/,/g;s/,$//" )
	JAHR=$( echo "${C}" | tr "\n" " " | sed -e "s/>/>\n/g" | grep -A 2 "Gründungsjahr" | tail -1l | sed -e "s/<[^>]*>//" )

	echo -n "${MA:-N/A};${EMAIL:-N/A};${GL:-N/A};${JAHR:-N/A};\"${DESC:-N/A}\";\"${CAT:-N/A}\""
}


S_COMP=$( echo "${COMP}" | sed -e "s/ /+/g;s/&/%26/g" )
#https://www.wlw.de/de/firmen?anzeige=firma&q=3Faktur+GmbH&q=3Faktur+GmbH&utf8=%E2%9C%93&filter%5Bcountry%5D%5B%5D=de&filter%5Bregion%5D=&filter%5Bradius%5D=50&filter%5Blat%5D=&filter%5Blng%5D=
A=$( wget -O - -q --user-agent="Research Bot v1.1" "https://www.wlw.de/de/firmen?anzeige=firma&q=3Faktur+GmbH&q=${S_COMP}&utf8=%E2%9C%93&filter%5Bcountry%5D%5B%5D=de&filter%5Bregion%5D=&filter%5Bradius%5D=50&filter%5Blat%5D=&filter%5Blng%5D=" | sed -e "s/>/>\n/g" )
if [ "${A}" == "" ]; then
	echo "Page ${i} not found" 
	break
fi

B=$( echo "${A}"| tr "\n" " " | sed -e "s/panel panel-sm/\npanel panel-sm/g;s/clickout_serp_company_url/\n/g" | grep "panel panel-sm" | grep -v "teaser-media" | grep "clickout" )

# Transform spaces to other character so that BASH for loop can
# iterate over
C=$( echo "${B}" | tr " " "£" | head -1l )

for b in ${C}; do
	B=$( echo "${b}" | tr "£" " " )
	COMPANY_ID=$( echo "${B}" | awk -F"\"" '{ print $11 }' )
	COMPANY_NAME=$( echo "${B}" | awk -F"\"" '{ print $43 }' | perl -C -MHTML::Entities -pe 'decode_entities($_);' | recode html...utf8 )
	ALT_NAME=$( echo "${B}" | awk -F"\"" '{ print $25 }' | perl -C -MHTML::Entities -pe 'decode_entities($_);' | recode html...utf8 )

	PLZ=$( echo "${B}" | awk -F">" '{ print $19 }' | sed -e "s/<\/li//;s/ D-//g;s/ DE-//g" )

	COMPANY_URL=$( echo "${B}" | awk -F"\"" '{ print $47 }' | awk -F"?" '{ print $1 }' | recode html...utf8 )

	WLW_URL=$( echo "${B}" | awk -F"\"" '{ print $77 }' | awk -F"?" '{ print $1 }' | sed -e "s/^/https:\/\/www.wlw.de/" | recode html...utf8)
	# Alteranative URL for the WLW information in case
	ALT_WLW_URL=$( echo "${B}" | awk -F"\"" '{ print $53 }' | awk -F"?" '{ print $1 }' | sed -e "s/^/https:\/\/www.wlw.de/" | recode html...utf8 )

	if [ "${COMPANY_URL}" != "non_customer" ] && [ "${COMPANY_URL}" != "customer" ] ; then
		if [ $( grep -c -m 1 "${COMPANY_NAME}" ${LOG} ) -eq 0 ]; then
			echo "[$( date +"%Y-%m-%d %H:%M:%S" )] Adding ${COMPANY_NAME}" > /dev/stderr

			DETAIL=$( detail_view "${WLW_URL}" )
			P_TITLE=$( html_page_title "${COMPANY_URL}" )
			P_TITLE=$( html_page_title "${COMPANY_URL}" )
			echo "${COMPANY_NAME};${COMPANY_URL};${PLZ};${COMPANY_ID};${WLW_URL};${DETAIL};${P_TITLE:-N/A}" | tee -a ${LOG}
		else
			P_TITLE=$( html_page_title "${COMPANY_URL}" )
			DETAIL=$( detail_view "${WLW_URL}" )
			echo "${COMPANY_NAME};${COMPANY_URL};${PLZ};${COMPANY_ID};${WLW_URL};${DETAIL};${P_TITLE:-N/A}"
		fi
	else
		if [ $( grep -c -m 1 "${ALT_NAME}" ${LOG} ) -eq 0 ]; then
			echo "[$( date +"%Y-%m-%d %H:%M:%S" )] ${ALT_NAME}" > /dev/stderr

			DETAIL=$( detail_view "${ALT_WLW_URL}" )
			ALT_URL=$( echo "${DETAIL}" | awk -F";" '{ print $2 }' | awk -F"@" '{ print $2 }' )

			P_TITLE=$( html_page_title "${ALT_URL}" )
			echo "${ALT_NAME};${ALT_URL};${PLZ};${COMPANY_ID};${ALT_WLW_URL};${DETAIL};${P_TITLE:-N/A}" | tee -a ${LOG}
		else
			DETAIL=$( detail_view "${ALT_WLW_URL}" )
			ALT_URL=$( echo "${DETAIL}" | awk -F";" '{ print $2 }' | awk -F"@" '{ print $2 }' )
			P_TITLE=$( html_page_title "${ALT_URL}" )
			echo "${ALT_NAME};${ALT_URL};${PLZ};${COMPANY_ID};${ALT_WLW_URL};${DETAIL};${P_TITLE:-N/A}"
		fi
	fi
done

echo "[$( date +"%Y-%m-%d %H:%M:%S" )] Now there are $( wc -l ${LOG} | awk '{ print $1 }' ) companies in ${LOG}" > /dev/stderr
